<?php
/**
 * This file handles the admin area and functions.
 *
 * @author LeapIn Corp.
 * @since 1.0.0
 */

/************* ADMIN NOTICE *****************/
function leapin_get_admin_notices()
{
    $ret = [/** since v 1.9.0 */
        'dashboard-message' => [
            'frequency' => 'once',
            'level' => 'info',
            'disp-in-pro' => true,
            'message' => sprintf(__('<b style="margin-right: 20px;">[Pjax Blog] Learn about Pjax Blog</b> <a class="button button-secondary" href=%s target="_blank">Initial Setup in 5 mins</a> <a class="button button-secondary" href=%s target="_blank">Document</a> <a class="button button-secondary" href=%s target="_blank">Customize</a> <a class="button button-secondary" href=%s target="_blank">Help</a> <a class="button button-primary" href=%s target="_blank">Upgrade to Pro</a>', 'pjax-blog')
                , esc_url('https://leap-in.com/installation/')
                , esc_url('https://leap-in.com/documents/')
                , leapin_get_customizer_sec_url('leapin_settings')
                , leapin_get_customizer_sec_url('leapin_help_section')
                , esc_url('https://leap-in.com/')
            )
        ],
    ];
    if (!leapin_is_bundle()) {
        $ret['install-bundle'] = [
            'frequency' => 'every-login',
            'level' => 'warning',
            'disp-in-pro' => false,
            'message' => sprintf(__('Please install free plugin <a href=%s target="_blank">Pjax Blog Bundle</a>! Make your site faster and powerful:)', 'pjax-blog'), esc_url('https://leap-in.com/plugin-pjax-blog-bundle/'))
        ];
    } else {
        if (!leapin_is_compatible_bundle_ver(LEAPIN_CAPABLE_BUNDLE, '>=')) {
            $ret['install-bundle-upgrade'] = [
                'frequency' => 'every-login',
                'level' => 'error',
                'disp-in-pro' => true,
                'message' => sprintf(__('Please update <a href=%s target="_blank">Pjax Blog Bundle</a> to v%s.', 'pjax-blog'), esc_url('https://leap-in.com/plugin-pjax-blog-bundle/'), LEAPIN_CAPABLE_BUNDLE)
            ];
        }
    }
    return $ret;
}

function leapin_add_document_url_to_admin_panel()
{
    $messages = leapin_get_admin_notices();
    // Only show this message on the admin dashboard and if asked for
    foreach ($messages as $id => $message) {
        leapin_the_dismissable_message($id, $message);
    }
}

add_action('admin_notices', 'leapin_add_document_url_to_admin_panel');


/**
 * Register dismissal of admin notices.
 *
 * Acts on the dismiss link in the admin nag messages.
 * If clicked, the admin notice disappears and will no longer be visible to this user until next login.
 *
 * @since 1.6.10
 */
function leapin_dismiss_notice()
{
    if (isset($_GET['leapin-dismiss']) && check_admin_referer('leapin-dismiss-' . get_current_user_id())) {
        update_user_meta(get_current_user_id(), 'leapin_dismissed_notice-' . esc_attr($_GET['leapin-dismiss']), 1);
    }
}

add_action('admin_head', 'leapin_dismiss_notice');
/**
 * Delete dismissable nag option when user login.
 *
 * This ensures that the user(s) is/are again reminded via nag of required
 * and/or recommended plugins if they re-activate the theme.
 *
 * @since 1.6.10
 */
function leapin_reset_dismiss_notice()
{
    $messages = leapin_get_admin_notices();
    foreach ($messages as $id => $val) {
        if ($val['frequency'] === 'every-login') {
            delete_metadata('user', null, 'leapin_dismissed_notice-' . $id, null, true);
        }
    }
}

add_action('wp_login', 'leapin_reset_dismiss_notice');

function leapin_the_dismissable_message($id, $message)
{
    $dismiss_endpoint = esc_url(wp_nonce_url(add_query_arg('leapin-dismiss', $id), 'leapin-dismiss-' . get_current_user_id()));
    $ms = $message['message'];
    $level = $message['level'];
    $disp_in_pro = $message['disp-in-pro'];
    if (!($disp_in_pro === false && leapin_is_pro())):
        if (!get_user_meta(get_current_user_id(), 'leapin_dismissed_notice-' . $id, 1)):
            ?>
            <div class="leapin notice notice-<?php echo $level; ?> is-dismissible"
                 data-dismiss-url="<?php echo $dismiss_endpoint; ?>">
                <p><?php echo $ms; ?></p>
            </div>
        <?php endif;
    endif;
}

function leapin_admin_wp_alpha_color_picker_field($id, $class = "", $name, $value, $show_opacity = 'true')
{
    printf(
        '<input id="%s" type="text" class="color-picker %s" name="%s" value="%s" data-alpha="%s" />',
        esc_attr($id),
        esc_attr($class),
        esc_attr($name),
        esc_attr($value),
        esc_attr($show_opacity)
    );
}


/**
 * Add custom checkbox field to widget
 *
 * @since 1.7.2
 * @param $widget_object
 * @param $instance
 * @param $field_id
 * @param $args
 *        label - input label
 */
function leapin_add_checkbox_to_widget_form($widget_object, $instance, $field_id, $args)
{
    if (!isset($args)) return;
    $label = array_key_exists('label', $args) ? $args['label'] : '';
    $value = array_key_exists($field_id, $instance) ? $instance[$field_id] : 0;
    ?>
    <p>
        <input id="<?php echo $widget_object->get_field_id($field_id); ?>"
               name="<?php echo $widget_object->get_field_name($field_id); ?>"
               type="checkbox" value="1" <?php checked($value, "1"); ?> />
        <label for="<?php echo $widget_object->get_field_id($field_id); ?>"><?php echo $label ?></label>
    </p>
    <?php
}

/**
 * Add custom number field to widget
 *
 * @since 1.7.2
 * @param $widget_object
 * @param $instance
 * @param $field_id widget field id
 * @param $args
 *        label - input label
 *        default - default value
 *        min - min number
 *        max - max number
 */
function leapin_add_number_to_widget_form($widget_object, $instance, $field_id, $args)
{
    if (!isset($args)) return;
    $label = array_key_exists('label', $args) ? $args['label'] : '';
    $default = array_key_exists('default', $args) ? $args['default'] : '';
    $min = array_key_exists('min', $args) ? sprintf(' min="%s"', $args['min']) : '';
    $max = array_key_exists('max', $args) ? sprintf(' max="%s"', $args['max']) : '';
    $current = isset($instance[$field_id]) || (array_key_exists($field_id, $instance) && $instance[$field_id] === 0)
        ? esc_attr($instance[$field_id]) : $default;
    ?>
    <p>
        <label for="<?php echo $widget_object->get_field_id($field_id); ?>"><?php echo $label; ?></label>
        <input class="tiny-text" id="<?php echo $widget_object->get_field_id($field_id); ?>"
               name="<?php echo $widget_object->get_field_name($field_id); ?>"<?php echo $min . $max; ?>
               type="number" value="<?php echo $current; ?>"/>
    </p>
    <?php
}

function leapin_add_text_to_widget_form($widget_object, $instance, $field_id, $args)
{
    if (!isset($args)) return;
    $label = array_key_exists('label', $args) ? $args['label'] : '';
    $type = array_key_exists('type', $args) ? $args['type'] : 'text';
    $default = array_key_exists('default', $args) ? $args['default'] : '';
    $current = isset($instance[$field_id]) || (array_key_exists($field_id, $instance) && $instance[$field_id] === 0)
        ? esc_attr($instance[$field_id]) : $default;
    ?>
    <p><label for="<?php echo $widget_object->get_field_id($field_id); ?>"><?php echo $label; ?></label>
        <input class="widefat" id="<?php echo $widget_object->get_field_id($field_id); ?>"
               name="<?php echo $widget_object->get_field_name($field_id); ?>" type="<?php echo $type; ?>"
               value="<?php echo $current; ?>"/>
    </p>
    <?php
}

function leapin_add_color_picker_to_widget_form($widget_object, $instance, $field_id, $args)
{
    if (!isset($args)) return;
    $label = array_key_exists('label', $args) ? $args['label'] : '';
    $default = array_key_exists('default', $args) ? $args['default'] : '';
    $show_opacity = array_key_exists('show_opacity', $args) && !empty($args['show_opacity']) ? "true" : "";
    $current = isset($instance[$field_id]) || (array_key_exists($field_id, $instance) && $instance[$field_id] === 0)
        ? esc_attr($instance[$field_id]) : $default;
    ?>
    <p>
        <label for="<?php echo $widget_object->get_field_id($field_id); ?>"
               style="display:block;"><?php echo $label; ?></label>
        <?php
        leapin_admin_wp_alpha_color_picker_field(
            $widget_object->get_field_id($field_id),
            "",
            $widget_object->get_field_name($field_id),
            $current,
            $show_opacity
        );
        ?>
    </p>
    <?php
}

function leapin_add_textarea_to_widget_form($widget_object, $instance, $field_id, $args)
{
    if (!isset($args)) return;
    $label = array_key_exists('label', $args) ? $args['label'] : '';
    $default = array_key_exists('default', $args) ? $args['default'] : '';
    $current = isset($instance[$field_id]) || (array_key_exists($field_id, $instance) && $instance[$field_id] === 0)
        ? esc_attr($instance[$field_id]) : $default;
    ?>
    <p>
        <label for="<?php echo $widget_object->get_field_id($field_id); ?>"><?php echo $label; ?></label>
        <textarea class="widefat" id="<?php echo $widget_object->get_field_id($field_id); ?>"
                  name="<?php echo $widget_object->get_field_name($field_id); ?>"><?php echo $current; ?></textarea>
    </p>
    <?php
}

function leapin_add_large_title_to_widget_form($title)
{
    ?>
    <p style="font-weight: 600;margin-top: 20px;font-size: 21px;"><?php echo $title; ?></p>
    <?php
}

/**
 * Add custom select field to widget
 *
 * @since 1.7.2
 * @param $widget_object
 * @param $instance
 * @param $field_id
 * @param $args
 *        label - select label
 *        default - default value
 *        add_empty - add '--' to the first option
 *        choice - options ['value' => 'label']
 */
function leapin_add_select_to_widget_form($widget_object, $instance, $field_id, $args)
{
    if (!isset($args)) return;
    $label = array_key_exists('label', $args) ? $args['label'] : '';
    $default = array_key_exists('default', $args) ? $args['default'] : '';
    $choice = array_key_exists('choice', $args) && is_array($args['choice']) ? $args['choice'] : [];
    $add_empty = array_key_exists('add_empty', $args) ? $args['add_empty'] : false;
    if ($add_empty) {
        $choice = array_merge(['' => '--'], $choice);
    }
    $current = isset($instance[$field_id]) ? $instance[$field_id] : $default;
    printf(
        '<p><label for="%1$s">%2$s</label><select class="widefat" id="%1$s" name="%3$s">',
        $widget_object->get_field_id($field_id),
        $label,
        $widget_object->get_field_name($field_id)
    );

    foreach ($choice as $value => $option_name) {
        printf(
            '<option value="%s"%s>%s</option>',
            esc_attr($value),
            selected($value, $current, false),
            $option_name
        );
    }
    echo '</select></p>';
}

function leapin_get_tax_color($tax_id_or_slug)
{
    global $leapin_vars;
    $taxonomies = $leapin_vars['taxonomies'];
    if (is_numeric($tax_id_or_slug)) {
        $t_id = $tax_id_or_slug;

    } else {
        if (array_key_exists($tax_id_or_slug, $taxonomies)) {
            $t_id = $taxonomies[$tax_id_or_slug]->ID;
        }
    }

    // Get the custom fields based on the $presenter term ID
    $custom_fields = get_option("taxonomy_term_$t_id");

    // Return the value for the "presenter_id" custom field
    $tax_color = $custom_fields['tax_color'];

    return $tax_color ? $tax_color : false;
}

function leapin_all_tax_color()
{
    global $leapin_vars;
    $taxonomies = $leapin_vars['taxonomies'];

    $tax_terms = [];
    foreach ($taxonomies as $tax => $tax_value) {
        $terms = get_terms(['taxonomy' => $tax, 'hide_empty' => false,]);
        foreach ($terms as $term) {
            $t_id = $term->term_id;
            // Get the custom fields based on the $presenter term ID
            $custom_fields = get_option("taxonomy_term_$t_id");

            // Return the value for the "presenter_id" custom field
            $tax_color = $custom_fields['tax_color'];

            $tax_color = $tax_color ? $tax_color : false;
            if ($tax_color) {
                $tax_terms[$term->term_id] = $tax_color;
            }
        }
    }
    return $tax_terms;
}


/*********************
 * SET CUSTOM FIELD TO TAX
 *********************/
function leapin_add_custom_field_to_taxonomy()
{
    // A callback function to add a custom field to our "colors" taxonomy
    function leapin_taxonomy_custom_fields($tag)
    {
        // Check for existing taxonomy meta for the term you're editing
        if (is_object($tag)) :
            $t_id = $tag->term_id; // Get the ID of the term you're editing
            $term_meta = get_option("taxonomy_term_$t_id"); // Do the check
            ?>
            <tr class="form-field">
                <th scope="row" valign="top">
                    <label for="tax_color"><?php _e('Theme color', 'pjax-blog'); ?></label>
                </th>
                <td>
                    <?php
                    leapin_admin_wp_alpha_color_picker_field(
                        '',
                        '',
                        'term_meta[tax_color]',
                        $term_meta['tax_color'] ? $term_meta['tax_color'] : '',
                        'true'
                    );
                    ?>
                    <p class="description"><?php _e('Picked color will be used in this theme.', 'pjax-blog'); ?></p>
                </td>
            </tr>
        <?php else: ?>
            <div class="form-field terms-tax_color-wrap">
                <label for="tax_color"><?php _e('Theme color', 'pjax-blog'); ?></label>
                <?php
                leapin_admin_wp_alpha_color_picker_field(
                    '',
                    'color-picker',
                    'term_meta[tax_color]',
                    '',
                    'true'
                );
                ?>
                <p class="description"><?php _e('Picked color will be used in this theme.', 'pjax-blog'); ?></p>
            </div>
        <?php endif;
    }

    // A callback function to save our extra taxonomy field(s)
    function leapin_save_taxonomy_custom_fields($term_id)
    {
        if (isset($_POST['term_meta'])) {
            $t_id = $term_id;
            $term_meta = get_option("taxonomy_term_$t_id");
            $cat_keys = array_keys($_POST['term_meta']);
            foreach ($cat_keys as $key) {
                if (isset($_POST['term_meta'][$key])) {
                    $term_meta[$key] = $_POST['term_meta'][$key];
                }
            }
            //save the option array
            update_option("taxonomy_term_$t_id", $term_meta);
        }
    }

    global $leapin_vars;
    $taxonomies = $leapin_vars['taxonomies'];

    foreach ($taxonomies as $tax) {
        $tax_name = $tax->name;
        // Add the fields to taxonomies, using our callback function
        add_action("${tax_name}_edit_form_fields", 'leapin_taxonomy_custom_fields', 10, 2);

        // Save the changes made on taxonomies, using our callback function
        add_action("edited_${tax_name}", 'leapin_save_taxonomy_custom_fields', 10, 2);
    }

}

// @todo implement in latter version
if (is_admin()) {
    add_action('wp_loaded', 'leapin_add_custom_field_to_taxonomy', 11);
}

?>
